import ast
import itertools
import unittest

from finder import Analyzer


class TestFinder(unittest.TestCase):
  def setUp(self):
    self.data = {
      -2: [
        "1.0",
        "1.",
        "float(x)",
        ],
      -1: [
        "1.1",
        "1+2j",
        "round(x, y)",
        ],
      0: [
        "x",
        ],
      1: [
        "1",
        "True",
        "False",
        "bool(x)",
        "int(x)",
        "len(x)",
        "round(x)",
        "not x",
        "~x",
        "x << y",
        "x >> y",
        "x | y",
        "x ^ y",
        "x & y",
        ],
      }
    for k, v in self.data.items():
      for expr in ["abs(%s)", "+%s", "-%s"]:
        v.append(expr % v[0])
    for type_a, type_b in itertools.product([-2, -1, 1], repeat=2):
      a, b = self.data[type_a][0], self.data[type_b][0]
      self.data[min(type_a, type_b)].append(a + " // " + b)
      if not type_a == type_b == 1:
        self.data[min(type_a, type_b)].append(a + " / " + b)

  def run_subtests(self, analyzer):
    for expected_result, expressions in self.data.items():
      for expr in expressions:
        with self.subTest(expr):
          tree = ast.parse(expr)
          result = analyzer.get_type(list(ast.walk(tree))[2])
          self.assertEqual(result, expected_result)

  def test_expressions_without_future_division(self):
    analyzer = Analyzer.__new__(Analyzer)
    self.data[1].extend([
      " / ".join([self.data[1][0]] * 2), # int / int
      ])
    self.run_subtests(analyzer)

  def test_expressions_with_future_division(self):
    analyzer = Analyzer.__new__(Analyzer)
    analyzer.future_division = True
    self.data[-1].extend([
      " / ".join([self.data[1][0]] * 2), # int / int
      ])
    self.run_subtests(analyzer)


if __name__ == '__main__':
  unittest.main()
